@@ -13,11 +13,11 @@
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2014
 
 from json import dumps, loads
-from random import choice
 from requests.status_codes import codes
 from urllib import quote
 
 from rucio.client.baseclient import BaseClient
+from rucio.client.baseclient import choice
 from rucio.common.utils import build_url
 
 
