@@ -7,7 +7,7 @@
 # Authors:
 # - Thomas Beermann, <thomas.beermann@cern.ch> , 2012
 # - Angelos Molfetas, <angelos.molfetas@cern,ch>, 2012
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012
+# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012, 2014
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2011-2013
 # - Ralph Vigne, <ralph.vigne@cern.ch>, 2012-2013
 # - Martin Barisits, <martin.barisits@cern.ch>, 2012-2014
@@ -84,6 +84,12 @@ class ClientProtocolNotSupported(RucioException):
         self._message = "Client protocol not supported."
 
 
+class ConfigNotFound(RucioException):
+    def __init__(self, *args, **kwargs):
+        super(ConfigNotFound, self).__init__(args, kwargs)
+        self._message = "Configuration not found."
+
+
 class ConfigurationError(RucioException):
     def __init__(self, *args, **kwargs):
         super(ConfigurationError, self).__init__(args, kwargs)
@@ -240,10 +246,10 @@ class InputValidationError(RucioException):
         self._message = "There is an error with one of the input parameters."
 
 
-class InsufficientQuota(RucioException):
+class InsufficientAccountLimit(RucioException):
     def __init__(self, *args, **kwargs):
-        super(InsufficientQuota, self).__init__(args, kwargs)
-        self._message = "There is not enough quota to fulfil the operation."
+        super(InsufficientAccountLimit, self).__init__(args, kwargs)
+        self._message = "There is not enough space left to fulfil the operation."
 
 
 class InsufficientTargetRSEs(RucioException):
